/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global console, window, tizen*/

/**
 * Common application model module.
 * It provides methods that use the Tizen Application API.
 *
 * @module app.common.application
 * @namespace app.common.application
 * @memberof app.common
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppCommonApplication(app) {
    'use strict';

    /**
     * Application object.
     *
     * @memberof app.common.application
     * @private
     * @type {ApplicationManager}
     */
    var application = null;

    /**
     * Launches application that allows the user to pick image.
     *
     * @memberof app.common.application
     * @public
     * @param {function} callback
     */
    function pickImage(callback) {
        var appControl = null,
            i = 0,
            length = 0;

        try {
            appControl = new tizen.ApplicationControl(
                'http://tizen.org/appcontrol/operation/pick',
                null,
                'image/*'
            );

            application.launchAppControl(
                appControl,
                null,
                null,
                function errorCallback(error) {
                    console.warn('Launch application control failed.', error);
                },
                {
                    onsuccess: function onPickImageSuccess(data) {
                        length = data.length;
                        for (i = 0; i < length; i += 1) {
                            if (data[i].key ===
                                'http://tizen.org/appcontrol/data/selected') {
                                callback(data[i].value[0]);
                                break;
                            }
                        }
                    }
                }
            );
        } catch (error) {
            console.warn('Launch application control failed.', error);
        }
    }

    /**
     * Launches application that allows the user to pick audio.
     *
     * @memberof app.common.application
     * @public
     * @param {function} callback
     */
    function pickAudio(callback) {
        var appControl = null,
            i = 0,
            length = 0;

        try {
            appControl = new tizen.ApplicationControl(
                'http://tizen.org/appcontrol/operation/pick',
                null,
                'audio/*'
            );

            application.launchAppControl(
                appControl,
                null,
                null,
                function errorCallback(error) {
                    console.warn('Launch application control failed.', error);
                },
                {
                    onsuccess: function onPickAudioSuccess(data) {
                        length = data.length;
                        for (i = 0; i < length; i += 1) {
                            if (data[i].key ===
                                'http://tizen.org/appcontrol/data/selected') {
                                callback(data[i].value[0]);
                                break;
                            }
                        }
                    }
                }
            );
        } catch (error) {
            console.warn('Launch application control failed.', error);
        }
    }

    /**
     * Initializes the common application model module.
     *
     * @memberof app.common.application
     * @public
     */
    function init() {
        application = tizen.application;
    }

    app.common = app.common || {};
    app.common.application = {
        init: init,
        pickImage: pickImage,
        pickAudio: pickAudio
    };

})(window.app);
