/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator option action module.
 * It is responsible for managing UI content,
 * displaying notification action option details.
 *
 * @module app.ui.creator.option.action
 * @requires {@link app.model.notification}
 * @requires {@link app.model.option}
 * @requires {@link app.ui.templates}
 * @namespace app.ui.creator.option.action
 * @memberof app.ui.creator.option
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreatorOptionAction(app) {
    'use strict';

    /**
     * UI templates module reference.
     *
     * @memberof app.ui.creator.option.action
     * @private
     * @type {object}
     */
    var uiTemplates = app.ui.templates,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator.option.action
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * Enter key code.
         *
         * @memberof app.ui.creator.option.action
         * @private
         * @type {number}
         */
        ENTER_KEYCODE = 13,

        /**
         * Parent container that contains option container.
         *
         * @memberof app.ui.creator.option.action
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Action input element.
         *
         * @memberof app.ui.creator.option.action
         * @private
         * @type {HTMLElement}
         */
        actionInput = null,

        /**
         * Action arrow element.
         *
         * @memberof app.ui.creator.option.action
         * @private
         * @type {HTMLElement}
         */
        actionArrow = null;

    /**
     * Handles blur event on action text input.
     *
     * @memberof app.ui.creator.option.action
     * @private
     */
    function onActionInputBlur() {
        var trimmedValue = actionInput.value.trim();

        actionInput.value = trimmedValue;
        modelNotification.setActionAppId(trimmedValue);
    }

    /**
     * Handles keydown event on action text input.
     *
     * @memberof app.ui.creator.option.action
     * @private
     * @param {Event} ev
     */
    function onActionInputKeydown(ev) {
        if (ev.keyCode === ENTER_KEYCODE) {
            ev.target.blur();
        }
    }

    /**
     * Handles click event on action arrow element.
     *
     * @memberof app.ui.creator.option.action
     * @private
     */
    function onActionArrowClick() {
        actionInput.focus();
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator.option.action
     * @private
     */
    function bindEvents() {
        actionInput.addEventListener('blur', onActionInputBlur);
        actionInput.addEventListener('keydown', onActionInputKeydown);
        actionArrow.addEventListener('click', onActionArrowClick);
    }

    /**
     * Shows action option content.
     *
     * @memberof app.ui.creator.option.action
     * @public
     */
    function show() {
        detailsContainer.innerHTML = uiTemplates.getActionOptionContent({
            actionApp: modelNotification.getActionAppId()
        });
        actionInput = detailsContainer.querySelector('.action-input');
        actionArrow = detailsContainer.querySelector('.action-arrow');
        bindEvents();
    }

    /**
     * Initializes the UI creator option action module.
     *
     * @memberof app.ui.creator.option.action
     * @public
     */
    function init() {
        detailsContainer = document.querySelector('.details');
    }

    app.ui = app.ui || {};
    app.ui.creator = app.ui.creator || {};
    app.ui.creator.option = app.ui.creator.option || {};
    app.ui.creator.option.action = {
        init: init,
        show: show
    };

})(window.app);
