/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator option led module.
 * It is responsible for managing UI content,
 * displaying notification led option details.
 *
 * @module app.ui.creator.option.led
 * @requires {@link app.common.events}
 * @requires {@link app.model.notification}
 * @requires {@link app.model.option}
 * @requires {@link app.ui.templates}
 * @namespace app.ui.creator.option.led
 * @memberof app.ui.creator.option
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreatorOptionLed(app) {
    'use strict';

    /**
     * UI templates module reference.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @type {object}
     */
    var uiTemplates = app.ui.templates,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * Common events module reference.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {object}
         */
        commonEvents = app.common.events,

        /**
         * Slider width in pixels.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @const {number}
         */
        SLIDER_WIDTH = 620,

        /**
         * Parent container that contains option container.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Light on handler.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {HTMLElement}
         */
        lightOnHandler = null,

        /**
         * Light on background.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {HTMLElement}
         */
        lightOnSlider = null,

        /**
         * Light off handler.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {HTMLElement}
         */
        lightOffHandler = null,

        /**
         * Light off background.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {HTMLElement}
         */
        lightOffSlider = null,

        /**
         * Color input.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {HTMLElement}
         */
        colorInput = null,

        /**
         * Currently used handler.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {HTMLElement}
         */
        currentHandler = null,

        /**
         * Currently used progress.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {HTMLElement}
         */
        currentProgress = null,

        /**
         * The value of the currently used progress.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {HTMLElement}
         */
        currentValue = null,

        /**
         * Start position of the cursor element before move.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {number}
         */
        handlerXStart = 0,

        /**
         * Left position of the handler element before move.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {number}
         */
        handlerLeftStart = 0,

        /**
         * The value equal to the sum of movement vectors for each moving step
         * of the handler element.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {number}
         */
        handlerXVectorSum = 0,

        /**
         * Previous difference between the cursor position
         * saved on touch start event on the handler element,
         * and saved while moving step.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {number}
         */
        handlerXDeltaBuffer = 0,

        /**
         * Max value of the LED on/off parameter.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {number}
         */
        maxLEDValue = 0,

        /**
         * Flag that indicates whether slider handler was moved.
         *
         * @memberof app.ui.creator.option.led
         * @private
         * @type {boolean}
         */
        wasHandlerMoved = false;

    /**
     * Maps slider value in pixels value to seconds.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {number} value
     * @returns {string}
     */
    function pixelsToSeconds(value) {
        var seconds = 0;

        if (value === SLIDER_WIDTH) {
            seconds = maxLEDValue;
        } else if (value > 0) {
            seconds = ((value * (maxLEDValue - 2) / SLIDER_WIDTH) + 1)
                .toFixed(0);
        }

        return seconds;
    }

    /**
     * Sets position of the currently used handler.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {number} value
     */
    function setHandlerPosition(value) {
        currentHandler.style.left = value + 'px';
    }

    /**
     * Sets width of the currently used progress.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {number} value
     */
    function setProgressWidth(value) {
        currentProgress.style.width = value + 'px';
    }

    /**
     * Sets value of the currently used slider.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {number} value
     */
    function setSliderValue(value) {
        currentValue.innerText = pixelsToSeconds(value);
    }

    /**
     * Updates slider value and appearance.
     *
     * @memberof app.ui.creator.option.led
     * @param {number} value
     */
    function updateSlider(value) {
        setHandlerPosition(value);
        setProgressWidth(value);
        setSliderValue(value);
    }

    /**
     * Handles touchstart event on handler element.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {Event} ev
     */
    function onHandlerTouchstart(ev) {
        // reset variables used between touchstart and touchend events
        handlerXDeltaBuffer = 0;
        handlerXVectorSum = 0;
        currentHandler = ev.target;
        currentProgress = currentHandler.parentNode.querySelector('.progress');
        currentValue = currentHandler.parentNode.querySelector('.value span');

        // save cursor start position
        handlerXStart = ev.touches[0].clientX;

        // save element left start position
        handlerLeftStart = currentHandler.offsetLeft;
    }

    /**
     * Handles touchmove event on handler element.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {Event} ev
     * @fires "change"
     */
    function onHandlerTouchmove(ev) {
        var handlerXDelta = 0,
            handlerXVector = 0,
            handlerLeftNew = 0;

        // set flag indicating that slider handler was moved to true
        wasHandlerMoved = true;

        // calculate difference between start position and
        // position after current movement
        handlerXDelta = ev.touches[0].clientX - handlerXStart;

        // calculate vector, which value is based on the current and previous
        // differences (between start position and position
        // after current movement)
        handlerXVector = handlerXDelta - handlerXDeltaBuffer;

        // calculate sum of vectors, which is equal to total shift
        // of the handler containers.
        handlerXVectorSum += handlerXVector;

        // calculate new left position of the handler element
        handlerLeftNew = handlerLeftStart + handlerXVectorSum;

        // limit available position of the handler
        handlerLeftNew = Math.max(0, Math.min(SLIDER_WIDTH, handlerLeftNew));

        // fire change event to notify about slider value change
        commonEvents.dispatchEvent(
            'change',
            {value: handlerLeftNew},
            ev.target.parentNode
        );

        // save value of the difference between start position and
        // position after current movement for the next movement
        handlerXDeltaBuffer = handlerXDelta;
    }

    /**
     * Handles touchstart event on background handler element.
     *
     * @memberof app.ui.creator.option.led
     * @private
     */
    function onSliderTouchstart() {
        wasHandlerMoved = false;
    }

    /**
     * Handles click event on background handler element.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {Event} ev
     * @fires "change"
     */
    function onSliderClick(ev) {
        var slider = ev.currentTarget,
            realLeftPosition = ev.clientX - 50;

        if (wasHandlerMoved) {
            return;
        }

        currentHandler = slider.querySelector('.handler');
        currentProgress = slider.querySelector('.progress');
        currentValue = slider.querySelector('.value span');

        if (realLeftPosition >= 0 && realLeftPosition <= SLIDER_WIDTH) {
            commonEvents.dispatchEvent(
                'change',
                {value: realLeftPosition},
                slider
            );
        }
    }

    /**
     * Handles input event on input color element.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {Event} ev
     */
    function onColorInput(ev) {
        modelNotification.setLEDColor(ev.target.value);
    }

    /**
     * Initializes max value of the LED on/off parameter.
     *
     * @memberof app.ui.creator.option.led
     * @private
     */
    function initMaxLEDValue() {
        maxLEDValue = modelNotification.getLEDMax();
    }

    /**
     * Handles change event on light on slider.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {Event} ev
     */
    function onLinghtOnSliderChange(ev) {
        var value = ev.detail.value;

        modelNotification.setLEDOn(value, pixelsToSeconds(value) * 1000);
        updateSlider(value);
    }

    /**
     * Handles change event on light off slider.
     *
     * @memberof app.ui.creator.option.led
     * @private
     * @param {Event} ev
     */
    function onLightOffSliderChange(ev) {
        var value = ev.detail.value;

        modelNotification.setLEDOff(value, pixelsToSeconds(value) * 1000);
        updateSlider(value);
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator.option.led
     * @private
     */
    function bindEvents() {
        lightOnHandler.addEventListener('touchstart', onHandlerTouchstart);
        lightOnHandler.addEventListener('touchmove', onHandlerTouchmove);
        lightOffHandler.addEventListener('touchstart', onHandlerTouchstart);
        lightOffHandler.addEventListener('touchmove', onHandlerTouchmove);
        lightOnSlider.addEventListener('touchstart', onSliderTouchstart);
        lightOffSlider.addEventListener('touchstart', onSliderTouchstart);
        lightOnSlider.addEventListener('click', onSliderClick);
        lightOffSlider.addEventListener('click', onSliderClick);
        lightOnSlider.addEventListener('change', onLinghtOnSliderChange);
        lightOffSlider.addEventListener('change', onLightOffSliderChange);
        colorInput.addEventListener('input', onColorInput);
    }

    /**
     * Shows led option content.
     *
     * @memberof app.ui.creator.option.led
     * @public
     */
    function show() {
        var ledOnValue = modelNotification.getLEDOn(),
            ledOffValue = modelNotification.getLEDOff();

        detailsContainer.innerHTML = uiTemplates.getLedOptionContent({
            ledOnSec: pixelsToSeconds(ledOnValue),
            ledOffSec: pixelsToSeconds(ledOffValue),
            ledOnWidth: ledOnValue,
            ledOffWidth: ledOffValue,
            ledOnLeft: ledOnValue,
            ledOffLeft: ledOffValue,
            ledColor: modelNotification.getLEDColor()
        });
        lightOnHandler = document.getElementById('light-on-handler');
        lightOffHandler = document.getElementById('light-off-handler');
        lightOnSlider = document.querySelector('.light-on-slider');
        lightOffSlider = document.querySelector('.light-off-slider');
        colorInput = document.querySelector('input[type="color"]');
        initMaxLEDValue();
        bindEvents();
    }

    /**
     * Initializes the UI creator option led module.
     *
     * @memberof app.ui.creator.option.led
     * @public
     */
    function init() {
        detailsContainer = document.querySelector('.details');
    }

    app.ui = app.ui || {};
    app.ui.creator = app.ui.creator || {};
    app.ui.creator.option = app.ui.creator.option || {};
    app.ui.creator.option.led = {
        init: init,
        show: show
    };

})(window.app);
