/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window*/

/**
 * The templates module.
 * Provides HTML templates for all pages used in the application.
 *
 * @module app.ui.templates
 * @requires {@link app.common.strings}
 * @namespace app.ui.templates
 * @memberof app.ui
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineTemplates(app) {
    'use strict';

    /**
     * Strings model module reference.
     *
     * @memberof app.ui.templates
     * @private
     * @type {object}
     */
    var commonStrings = app.common.strings,

        /**
         * Four spaces gap.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        TAB = '&nbsp;&nbsp;&nbsp;&nbsp;',

        /**
         * Template for indicator element.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        INDICATOR_ELEMENT_TEMPLATE = '<div class="indicator-element"></div>',

        /**
         * Template for basic option content.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        BASIC_OPTION_CONTENT_TEMPLATE =
            '<div class="option-content basic-option">' +
                '<div class="title-container">' +
                    '<input class="input title-input"' +
                        'placeholder="Insert title ..." value="%title%">' +
                    '<div class="title-indicator"></div>' +
                    '<div class="title-message">' +
                        'Please be informed that this field is required' +
                    '</div>' +
                '</div>' +
                '<input class="input content-input"' +
                    'placeholder="Insert content ..." value="%content%">' +
            '</div>',

        /**
         * Template for icons option content.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        ICONS_OPTION_CONTENT_TEMPLATE =
            '<div class="option-content icons-option">' +
                '<div class="main-icon"' +
                    'style="background-image: url(&quot;%icon%&quot;)">' +
                    '<div class="sub-icon"' +
                        'style="background-image:' +
                        'url(&quot;%subicon%&quot;)"></div>' +
                '</div>' +
                '<div class="pointer icon-pointer"></div>' +
                '<div class="label icon-label">' +
                    'tap to add<br/>main icon' +
                '</div>' +
                '<div class="pointer subicon-pointer"></div>' +
                '<div class="label subicon-label">' +
                    'tap to add<br/>sub icon' +
                '</div>' +
                '<div class="icon-message">' +
                    'You don\'t have to add both icons' +
                '</div>' +
            '</div>',

        /**
         * Template for repeat option content.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        REPEAT_OPTION_CONTENT_TEMPLATE =
            '<div class="option-content repeat-option">' +
                '<div class="button decrease-btn"></div>' +
                '<div class="bell">' +
                    '<div class="value">%repeat%</div>' +
                '</div>' +
                '<div class="button increase-btn"></div>' +
                '<div class="message">' +
                    'The maximum number of event is 20' +
                '</div>' +
            '</div>',

        /**
         * Template for LED option content.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        LED_OPTION_CONTENT_TEMPLATE =
            '<div class="option-content led-option">' +
                '<input type="color" value="%ledColor%">' +
                '<div class="led-arrow"></div>' +
                '<div class="led-label">' +
                    'tap icon to choose color for LED' +
                '</div>' +
                '<div class="slider light-on-slider">' +
                    '<div class="background" id="light-on-background"></div>' +
                    '<div class="progress"' +
                        'style="width: %ledOnWidth%px"></div>' +
                    '<div class="handler" id="light-on-handler"' +
                        'style="left: %ledOnLeft%px"></div>' +
                    '<div class="text label">Light On</div>' +
                    '<div class="text value">' +
                        '<span>%ledOnSec%</span> sec' +
                    '</div>' +
                '</div>' +
                '<div class="slider light-off-slider">' +
                    '<div class="background" id="light-off-background"></div>' +
                    '<div class="progress"' +
                        'style="width: %ledOffWidth%px"></div>' +
                    '<div class="handler" id="light-off-handler"' +
                        'style="left: %ledOffLeft%px"></div>' +
                    '<div class="text label">Ligth Off</div>' +
                    '<div class="text value">' +
                        '<span>%ledOffSec%</span> sec' +
                    '</div>' +
                '</div>' +
            '</div>',

        /**
         * Template for sound option content.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        SOUND_OPTION_CONTENT_TEMPLATE =
            '<div class="option-content sound-option">' +
                '<div class="sound-icon" id="sound-icon"></div>' +
                '<div class="sound-url">%audioURL%</div>' +
                '<div class="sound-arrow"></div>' +
                '<div class="sound-label">' +
                    'tap on the icon<br/>to pick audio file' +
                '</div>' +
                '<div class="sound-message">' +
                    'You can load any audio file format' +
                '</div>' +
            '</div>',

        /**
         * Template for vibration option content.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        VIBRATION_OPTION_CONTENT_TEMPLATE =
            '<div class="option-content vibration-option %vibrationEnabled%">' +
                '<div class="button vibration-on"' +
                    'id="vibration-on-btn"></div>' +
                '<div class="vibration-icon"></div>' +
                '<div class="button vibration-off"' +
                    'id="vibration-off-btn"></div>' +
                '<div class="vibration-message">' +
                    'Vibration works independently<br/>' +
                    'of the sound option settings' +
                '</div>' +
            '</div>',

        /**
         * Template for progress option content.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        PROGRESS_OPTION_CONTENT_TEMPLATE =
            '<div class="option-content progress-option %progressType%">' +
                '<div class="button percentage" id="percentage-btn"></div>' +
                '<div class="button byte" id="byte-btn"></div>' +
                '<div class="progress-message">' +
                    'You can choose the type<br/>' +
                    'of the progress notification' +
                '</div>' +
            '</div>',

        /**
         * Template for details option content.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        DETAILS_OPTION_CONTENT_TEMPLATE =
            '<div class="option-content details-option">' +
                '<input class="input main-input"' +
                    'placeholder="main text ..." value="%mainText%">' +
                '<input class="input sub-input"' +
                    'placeholder="sub text ..." value="%subText%">' +
                '<div class="details-icon"></div>' +
                '<div class="details-message">' +
                    'These two fields can be displayed in the top bar<br/>' +
                    'and are optional' +
                '</div>' +
            '</div>',

        /**
         * Template for action option content.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        ACTION_OPTION_CONTENT_TEMPLATE =
            '<div class="option-content action-option">' +
                '<div class="action-label">' +
                    '<input class="input action-input"' +
                        'placeholder="enter app ID you want to launch ..."' +
                        'value="%actionApp%">' +
                    '<div class="action-arrow"></div>' +
                '</div>' +
                '<div class="action-message">' +
                    'Be sure that entered application ID is valid<br/>' +
                '</div>' +
            '</div>',

        /**
         * Template for code variables.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_VARIABLES = 'var status = null;<br/><br/>',

        /**
         * Template for code try.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_TRY = 'try {<br/>',

        /**
         * Template for code catch.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_CATCH = '} catch (error) {<br/>' +
            TAB + 'console.warn(\'Send operation failed.\', error);<br/>' +
            '}<br/>',

        /**
         * Template for code status begin.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_STATUS_BEGIN = TAB + 'status = new tizen.StatusNotification(<br/>',

        /**
         * Template for code status end.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_STATUS_END = TAB + ');<br/>',

        /**
         * Template for code notification type.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_NOTIFICATION_TYPE = TAB + TAB + '\'%type%\',<br/>',

        /**
         * Template for code notification title.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_NOTIFICATION_TITLE = TAB + TAB + '\'%title%\',<br/>',

        /**
         * Template for code data begin.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_DATA_BEGIN = TAB + TAB + '{',

        /**
         * Template for code data end.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_DATA_END = '<br/>' + TAB + TAB + '}<br/>',

        /**
         * Template for code notification attribute which value is not a string.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_NOTIFICATION_ATTRIBUTE = '<br/>' + TAB + TAB + TAB +
            '%attribute%: %value%,',

        /**
         * Template for code notification attribute which value is a string.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_NOTIFICATION_ATTRIBUTE_STRING = '<br/>' + TAB + TAB + TAB +
            '%attribute%: \'%value%\',',

        /**
         * Template for code detail.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_DETAIL = '<br/>' + TAB + TAB + TAB +
            'detailInfo: [new tizen.NotificationDetailInfo(%value%)],',

        /**
         * Template for code send.
         *
         * @memberof app.ui.templates
         * @private
         * @const {string}
         */
        CODE_SEND = TAB + 'tizen.notification.post(status);<br/>';

    /**
     * Returns HTML for indicator element.
     *
     * @memberof app.ui.templates
     * @public
     * @returns {string}
     */
    function getIndicatorElement() {
        return INDICATOR_ELEMENT_TEMPLATE;
    }

    /**
     * Returns HTML for basic option content.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.title Notification title.
     * @param {string} data.content Notification content.
     * @returns {string}
     */
    function getBasicOptionContent(data) {
        return BASIC_OPTION_CONTENT_TEMPLATE
            .replace('%title%', commonStrings.escapeHTML(data.title))
            .replace('%content%', commonStrings.escapeHTML(data.content));
    }

    /**
     * Returns HTML for basic option content.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.iconURL Notification icon URL.
     * @param {string} data.subiconURL Notification subicon URL.
     * @returns {string}
     */
    function getIconsOptionContent(data) {
        return ICONS_OPTION_CONTENT_TEMPLATE
            .replace('%icon%', commonStrings.escapeHTML(data.iconURL))
            .replace('%subicon%', commonStrings.escapeHTML(data.subiconURL));
    }

    /**
     * Returns HTML for repeat option content.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {number} data.repeat Repeat parameter value.
     * @returns {string}
     */
    function getRepeatOptionContent(data) {
        return REPEAT_OPTION_CONTENT_TEMPLATE
            .replace('%repeat%', data.repeat);
    }

    /**
     * Returns HTML for LED option content.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {number} data.ledOnSec LED on parameter value (in seconds).
     * @param {number} data.ledOffSec LED off parameter value (in seconds).
     * @param {number} data.ledOnWidth LED on progress element width.
     * @param {number} data.ledOffWidth LED off progress element width.
     * @param {number} data.ledOnLeft LED on handler element left value.
     * @param {number} data.ledOffLeft LED off handler element left value.
     * @param {string} data.ledColor LED color value.
     * @returns {string}
     */
    function getLedOptionContent(data) {
        return LED_OPTION_CONTENT_TEMPLATE
            .replace('%ledOnSec%', data.ledOnSec)
            .replace('%ledOffSec%', data.ledOffSec)
            .replace('%ledOnWidth%', data.ledOnWidth)
            .replace('%ledOffWidth%', data.ledOffWidth)
            .replace('%ledOnLeft%', data.ledOnLeft)
            .replace('%ledOffLeft%', data.ledOffLeft)
            .replace('%ledColor%', commonStrings.escapeHTML(data.ledColor));
    }

    /**
     * Returns HTML for sound option content.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.audioURL URL of the notification audio.
     * @returns {string}
     */
    function getSoundOptionContent(data) {
        return SOUND_OPTION_CONTENT_TEMPLATE
            .replace('%audioURL%', commonStrings.escapeHTML(data.audioURL));
    }

    /**
     * Returns HTML for vibration option content.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {boolean} data.vibrationEnabled Notification vibration state.
     * @returns {string}
     */
    function getVibrationOptionContent(data) {
        return VIBRATION_OPTION_CONTENT_TEMPLATE
            .replace('%vibrationEnabled%', data.vibrationEnabled ? 'on' : '');
    }

    /**
     * Returns HTML for progress option content.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.progressType Notification progress type.
     * @returns {string}
     */
    function getProgressOptionContent(data) {
        var type = data.progressType;

        type = (type === 'BYTE') ? type.toLowerCase() : '';

        return PROGRESS_OPTION_CONTENT_TEMPLATE
            .replace('%progressType%', commonStrings.escapeHTML(type));
    }

    /**
     * Returns HTML for details option content.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.mainText Notification main details value.
     * @param {string} data.subText Notification sub details value.
     * @returns {string}
     */
    function getDetailsOptionContent(data) {
        return DETAILS_OPTION_CONTENT_TEMPLATE
            .replace('%mainText%', commonStrings.escapeHTML(data.mainText))
            .replace('%subText%', commonStrings.escapeHTML(data.subText));
    }

    /**
     * Returns HTML for action option content.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.actionApp Notification action app ID value.
     * @returns {string}
     */
    function getActionOptionContent(data) {
        return ACTION_OPTION_CONTENT_TEMPLATE
            .replace('%actionApp%', commonStrings.escapeHTML(data.actionApp));
    }

    /**
     * Returns HTML for code variables.
     *
     * @memberof app.ui.templates
     * @public
     * @returns {string}
     */
    function getCodeVariables() {
        return CODE_VARIABLES;
    }

    /**
     * Returns HTML for code try.
     *
     * @memberof app.ui.templates
     * @public
     * @returns {string}
     */
    function getCodeTry() {
        return CODE_TRY;
    }

    /**
     * Returns HTML for code catch.
     *
     * @memberof app.ui.templates
     * @public
     * @returns {string}
     */
    function getCodeCatch() {
        return CODE_CATCH;
    }

    /**
     * Returns HTML for code status begin.
     *
     * @memberof app.ui.templates
     * @public
     * @returns {string}
     */
    function getCodeStatusBegin() {
        return CODE_STATUS_BEGIN;
    }

    /**
     * Returns HTML for code status end.
     *
     * @memberof app.ui.templates
     * @public
     * @returns {string}
     */
    function getCodeStatusEnd() {
        return CODE_STATUS_END;
    }

    /**
     * Returns HTML for code notification type.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.type Notification type.
     * @returns {string}
     */
    function getCodeNotificationType(data) {
        return CODE_NOTIFICATION_TYPE
            .replace('%type%', commonStrings.escapeHTML(data.type));
    }

    /**
     * Returns HTML for code notification title.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.title Notification title.
     * @returns {string}
     */
    function getCodeNotificationTitle(data) {
        return CODE_NOTIFICATION_TITLE
            .replace('%title%', commonStrings.escapeHTML(
                commonStrings.escapeSingleQuote(data.title)
            ));
    }

    /**
     * Returns HTML for code data begin.
     *
     * @memberof app.ui.templates
     * @public
     * @returns {string}
     */
    function getCodeDataBegin() {
        return CODE_DATA_BEGIN;
    }

    /**
     * Returns HTML for code data end.
     *
     * @memberof app.ui.templates
     * @public
     * @returns {string}
     */
    function getCodeDataEnd() {
        return CODE_DATA_END;
    }

    /**
     * Returns HTML for code detail.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.mainDetails Notification main details value.
     * @param {string} data.subDetails Notification sub details value.
     * @returns {string}
     */
    function getCodeDetail(data) {
        var mainDetails = commonStrings.escapeHTML(
                commonStrings.escapeSingleQuote(data.mainDetails)
            ),
            subDetails = '',
            allDetails = '\'' + mainDetails + '\'';

        if (data.subDetails) {
            subDetails = commonStrings.escapeHTML(
                commonStrings.escapeSingleQuote(data.subDetails)
            );
            allDetails += ', \'' + subDetails + '\'';
        }

        return CODE_DETAIL.replace('%value%', allDetails);
    }

    /**
     * Returns HTML for code send.
     *
     * @memberof app.ui.templates
     * @public
     * @returns {string}
     */
    function getCodeSend() {
        return CODE_SEND;
    }

    /**
     * Returns HTML for code notification attribute which value is not a string.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.attribute Notification attribute name.
     * @param {string} data.value Notification attribute value.
     * @returns {string}
     */
    function getCodeAttribute(data) {
        return CODE_NOTIFICATION_ATTRIBUTE
            .replace('%attribute%', commonStrings.escapeHTML(data.attribute))
            .replace('%value%', commonStrings.escapeHTML(data.value));
    }

    /**
     * Returns HTML for code notification attribute which value is a string.
     *
     * @memberof app.ui.templates
     * @public
     * @param {object} data
     * @param {string} data.attribute Notification attribute name.
     * @param {string} data.value Notification attribute value.
     * @returns {string}
     */
    function getCodeAttributeString(data) {
        return CODE_NOTIFICATION_ATTRIBUTE_STRING
            .replace('%attribute%', commonStrings.escapeHTML(data.attribute))
            .replace('%value%', commonStrings.escapeHTML(
                commonStrings.escapeSingleQuote(data.value)
            ));
    }

    app.ui = app.ui || {};
    app.ui.templates = {
        getIndicatorElement: getIndicatorElement,
        getBasicOptionContent: getBasicOptionContent,
        getIconsOptionContent: getIconsOptionContent,
        getRepeatOptionContent: getRepeatOptionContent,
        getLedOptionContent: getLedOptionContent,
        getSoundOptionContent: getSoundOptionContent,
        getVibrationOptionContent: getVibrationOptionContent,
        getProgressOptionContent: getProgressOptionContent,
        getDetailsOptionContent: getDetailsOptionContent,
        getActionOptionContent: getActionOptionContent,
        getCodeVariables: getCodeVariables,
        getCodeTry: getCodeTry,
        getCodeCatch: getCodeCatch,
        getCodeStatusBegin: getCodeStatusBegin,
        getCodeStatusEnd: getCodeStatusEnd,
        getCodeNotificationType: getCodeNotificationType,
        getCodeNotificationTitle: getCodeNotificationTitle,
        getCodeDataBegin: getCodeDataBegin,
        getCodeDataEnd: getCodeDataEnd,
        getCodeAttribute: getCodeAttribute,
        getCodeAttributeString: getCodeAttributeString,
        getCodeDetail: getCodeDetail,
        getCodeSend: getCodeSend
    };

})(window.app);
